
#include "inc.h"

#include "minix.h"
#include "irq_arm.h"
#include "bios_arm.h"
#include "div_arm.h"
#include "ad_arm.h"
#include "gbfs.h"
#include "lib.h"

#include "screen.h"
#include "sudoku.h"
#include "main.h"

//#include "header\font\BGFont.h"


//Mode3

//---------------------------------------------------------------------------
//lib.c
extern ST_FONT Font;

//---------------------------------------------------------------------------
ST_SELECT Select;

u8 State;
//---------------------------------------------------------------------------
EWRAM_CODE int main()
{
	State = STATE_INIT;

	for(;;)
	{
		switch(State)
		{
			case STATE_INIT:       StateInit();      break;
			case STATE_RESET:      StateReset();     break;
			case STATE_SELECT:     StateSelect();    break;
			case STATE_GAME_INPUT: StateGameInput(); break;
			case STATE_GAME_CHK:   StateGameChk();   break;
			case STATE_GAME_END:   StateGameEnd();   break;

			default:
				DebugMessage("[Err] main State");
				break;
		}
	}


	return 0;
}
//---------------------------------------------------------------------------
EWRAM_CODE void StateInit()
{
	InitWS();
	InitLib();
	InitKey();
	InitIRQ();
	InitFont();
	InitSprite();

	SetMode(MODE_3 | BG2_ENABLE | OBJ_ENABLE | OBJ_MAP_2D);

	State = STATE_RESET;
}
//---------------------------------------------------------------------------
EWRAM_CODE void InitWS()
{
	REG_WSCNT = 0x4317;
}
//---------------------------------------------------------------------------
EWRAM_CODE void InitLib()
{
	LibInit();
}
//---------------------------------------------------------------------------
EWRAM_CODE void InitKey()
{
	KeyInit();
}
//---------------------------------------------------------------------------
EWRAM_CODE void InitIRQ()
{
	REG_INTERUPT = (u32)IRQUserHandler;
	REG_IE       = IRQ_BIT_TIMER2;

	//L[̃s[gԗp
	REG_TM2D     = 65536 - 2621;
	REG_TM2CNT   = TM_ENABLE | TM_FREQ_PER_64 | TM_USEIRQ;

	REG_IME      = IRQ_MASTER_ON;
}
//---------------------------------------------------------------------------
EWRAM_CODE void InitFont()
{
	ST_FONT_SRC half;		//p

	half.pDat   = (u8*)&mplus_s10rBitmap;
	half.pSheet = (u16*)&mplus_sfnt_txt;
	half.imgCx  = MPLUS_S10R_IMG_CX;
	half.cnt    = MPLUS_S10R_FONT_CNT;
	half.cx     = MPLUS_S10R_FONT_CX;
	half.cy     = MPLUS_S10R_FONT_CY;

	Mode3SetFont(NULL, &half);
}
//---------------------------------------------------------------------------
EWRAM_CODE void InitSprite()
{
	SpriteInit();

	SpriteSetPalData16((u16*)&sprPal);
	SpriteSetData((u16*)&sprTiles, sprTilesLen);
}
//---------------------------------------------------------------------------
EWRAM_CODE void StateReset()
{
	ScreenImgInit();
	ScreenCurInit();

	SudokuInit();
	SudokuDrawBg();

	SelectInit();
	SelectLoad();

	State = STATE_SELECT;
}
//---------------------------------------------------------------------------
EWRAM_CODE void StateSelect()
{
	SelectProcMenu();
	SelectProcSolve();

	ScreenCurMoveIn(CELL_SX, CELL_SY);

	State = STATE_GAME_INPUT;
}
//---------------------------------------------------------------------------
EWRAM_CODE void StateGameInput()
{
	ScreenCurInput();

	State = STATE_GAME_CHK;
}
//---------------------------------------------------------------------------
EWRAM_CODE void StateGameChk()
{
	State = ( SudokuIsGameEnd() == TRUE ) ? STATE_GAME_END : STATE_GAME_INPUT;
}
//---------------------------------------------------------------------------
EWRAM_CODE void StateGameEnd()
{
	SelectDrawMsgEnd();
	KeyWait(KEY_START);

	State = STATE_RESET;
}




//---------------------------------------------------------------------------
EWRAM_CODE void SelectInit()
{
	_Memset((u8*)&Select, 0x00, sizeof(ST_SELECT));
}
//---------------------------------------------------------------------------
EWRAM_CODE void SelectLoad()
{
	ST_SELECT* p = &Select;

	p->pHead  = (u8*)&data_txt;
	p->pCur   = (u8*)&data_txt;
	p->size   = data_txt_size;
	p->maxNum = _UDiv(data_txt_size, SUDOKU_QUIZ_SIZE) - 1;
	p->num    = 0;
}
//---------------------------------------------------------------------------
EWRAM_CODE void SelectDrawMsgChoice()
{
	ST_SELECT* p = &Select;
	char buf[64];

	_Sprintf(buf, "Select: %05d/%05d", p->num+1, p->maxNum+1);
	Mode3DrawFontBackGround(0, 0, (u8*)buf, COLOR_WHITE, COLOR_BLACK);

	ScreenImgUpdate();
}
//---------------------------------------------------------------------------
EWRAM_CODE void SelectDrawMsgSolve()
{
	Mode3DrawFontBackGround(0, 0, (u8*)"Make Solve....", COLOR_WHITE, COLOR_BLACK);
	ScreenImgUpdate();
}
//---------------------------------------------------------------------------
EWRAM_CODE void SelectDrawMsgEnd()
{
	Mode3DrawFontBackGround(0, 0, (u8*)"Game Clear!", COLOR_WHITE, COLOR_BLACK);
	ScreenImgUpdate();
}
//---------------------------------------------------------------------------
EWRAM_CODE void SelectDrawMsgClear()
{
	Mode3DrawFillBox(0, 0, 160, 11, COLOR_BLACK);
	ScreenImgUpdate();
}
//---------------------------------------------------------------------------
EWRAM_CODE void SelectProcMenu()
{
	ST_SELECT* p = &Select;
	u16 key;

	SelectProcBoard();
	SelectDrawMsgChoice();

	for(;;)
	{
		WaitForVsync();
		key = KeyGet();

		if(key == KEY_NONE)
		{
			continue;
		}

		if(key & KEY_A)
		{
			break;
		}
		else if(key & KEY_LEFT)
		{
			p->num = MAX(p->num-1, 0);
		}
		else if(key & KEY_DOWN)
		{
			p->num = MAX(p->num-100, 0);
		}
		else if(key & KEY_RIGHT)
		{
			p->num = MIN(p->num+1, p->maxNum);
		}
		else if(key & KEY_UP)
		{
			p->num = MIN(p->num+100, p->maxNum);
		}

		SelectProcBoard();
		SelectDrawMsgChoice();
	}

	KeyWait3();
	SelectDrawMsgClear();
}
//---------------------------------------------------------------------------
EWRAM_CODE void SelectProcBoard()
{
	ST_SELECT* p = &Select;

	p->pCur  = p->pHead;
	p->pCur += p->num * SUDOKU_QUIZ_SIZE;

	SudokuLoadStage(p->pCur);
	SudokuDrawCellAll();
}
//---------------------------------------------------------------------------
EWRAM_CODE void SelectProcSolve()
{
	SelectDrawMsgSolve();
	SudokuProcSolve();

	SelectDrawMsgClear();
}




//---------------------------------------------------------------------------
EWRAM_CODE void SystemError(char* msg1, char* msg2)
{
	LibMode3SetScreenBuffer((u16*)VideoBuffer);

	if(msg1 != NULL) Mode3DrawFontStr(0,  0, (u8*)msg1, COLOR_WHITE);
	if(msg2 != NULL) Mode3DrawFontStr(0, 11, (u8*)msg2, COLOR_WHITE);

	for(;;){}
}
