
#include "inc.h"

#include "minix.h"
#include "irq_arm.h"
#include "bios_arm.h"
#include "div_arm.h"
#include "gbfs.h"
#include "lib.h"

#include "sudoku.h"
#include "screen.h"

//---------------------------------------------------------------------------
ST_SCREEN_IMG ScreenImg EWRAM_DATA;
ST_CURSOR     Cursor;

u32 dummy ALIGN(4);
//---------------------------------------------------------------------------
EWRAM_CODE void ScreenImgInit()
{
	LibMode3SetScreenBuffer((u16*)&ScreenImg.buf);

	dummy = 0;
	ScreenImgClear();
	ScreenImgUpdate();
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScreenImgClear()
{
	CpuSet((u32)&dummy, (u32)ScreenImg.buf, ((240*160*2) / 4) | CPUSET_32BIT | CPUSET_SRC_FIX);
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScreenImgUpdate()
{
	WaitForVsync();
	CpuSet((u32)ScreenImg.buf, (u32)VideoBuffer, ((240*160*2) / 4) | CPUSET_32BIT);
}





//---------------------------------------------------------------------------
EWRAM_CODE void ScreenCurInit()
{
	ST_CURSOR* p = &Cursor;

	p->x      = 0;
	p->y      = 0;
	p->cx     = 16;
	p->cy     = 16;
	p->sx     = CELL_SX;
	p->sy     = CELL_SY;
	p->ex     = CELL_SX + CELL_CX * 8;
	p->ey     = CELL_SY + CELL_CY * 8;
	p->sprNo  = CURSOR_SPRITE_NO;
	p->tileNo = CURSOR_TILE_NO_RECT;

	SpriteSetSize(p->sprNo, SP_SIZE_16, SP_SQUARE, SP_COLOR_16);
	SpriteSetChr(p->sprNo, p->tileNo);

	ScreenCurMoveOut();
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScreenCurMove()
{
	ST_CURSOR* p = &Cursor;
	SpriteMove(p->sprNo, p->x, p->y);
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScreenCurMoveIn(s16 x, s16 y)
{
	ST_CURSOR* p = &Cursor;
	p->x = x;
	p->y = y;

	SpriteMove(p->sprNo, p->x, p->y);
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScreenCurMoveOut()
{
	ST_CURSOR* p = &Cursor;
	SpriteMove(p->sprNo, 0, SCREEN_CY);
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScreenCurInput()
{
	ST_CURSOR* p = &Cursor;
	u16 x, y;
	u16 key;

	for(;;)
	{
		WaitForVsync();
		key = KeyGet();

		if(key == KEY_NONE)
		{
			continue;
		}

		if(key & KEY_A)
		{
			x = _UDiv(p->x - CELL_SX, CELL_CX);
			y = _UDiv(p->y - CELL_SY, CELL_CY);

			SudokuChgCell(x, y, 1);
			return;
		}
		else if(key & KEY_B)
		{
			x = _UDiv(p->x - CELL_SX, CELL_CX);
			y = _UDiv(p->y - CELL_SY, CELL_CY);

			SudokuChgCell(x, y, -1);
			return;
		}

		if(key & KEY_UP)
		{
			p->y = MAX(p->y - p->cy, p->sy);
		}
		else if(key & KEY_DOWN)
		{
			p->y = MIN(p->y + p->cy, p->ey);
		}
		else if(key & KEY_LEFT)
		{
			p->x = MAX(p->x - p->cx, p->sx);
		}
		else if(key & KEY_RIGHT)
		{
			p->x = MIN(p->x + p->cx, p->ex);
		}

		ScreenCurMove();
	}
}
